import pathlib
import shutil
import subprocess

import requests

from git_cloner.utils.config import Config


def list_repos(org, limit):
    print(limit)
    limit = int(limit)
    total_pages = int(limit / int(Config.get_config().limit_per_page)) + 1
    if limit % Config.get_config().limit_per_page:
        total_pages += 1
    print(total_pages)
    return [repo for page in range(1, total_pages) for repo in list_repos_page(org, page)]


def list_repos_page(org, page):
    KEYS = ['name', 'clone_url', 'ssh_url']
    auth = (Config.get_config().user, Config.get_config().password) if Config.get_config().user else None
    url = 'https://api.github.com/orgs/{org}/repos?page={page}&per_page={limit}'.format(
        org=org, limit=Config.get_config().limit_per_page, page=page)
    r = requests.get(url, auth=auth)
    if r.status_code != 200:
        print("Error {} in the request".format(r.status_code))
        return []

    repos = r.json()
    filter_repos = [
        {key: repo.get(key) for key in KEYS}
        for repo in repos]
    print(len(filter_repos))
    return filter_repos


def clones(repos):
    out_dir = '.' if Config.get_config().dir_out is None else Config.get_config().dir_out
    force = Config.get_config().force
    size = len(repos)

    repo_result = []
    if out_dir is not None:
        pathlib.Path(out_dir).mkdir(parents=True, exist_ok=True)
    print("Cloning repositories {}".format(len(repos)))
    n = 1
    for repo in repos:
        repo_dir = "{}/{}".format(out_dir, repo.get("name"))
        repo_name = repo.get("name")
        clone_url = repo.get('ssh_url') if Config.get_config().ssh else repo.get('clone_url')
        print("Clone repository {name} ({n}/{total})".format(n=n, total=size, name=repo_name))
        cmd_str = 'git clone {clone_url} {dir}'.format(clone_url=clone_url, dir=repo_dir)
        if force:
            shutil.rmtree(repo_dir, ignore_errors=True)
        cmd = cmd_str.split(" ")
        try:
            subprocess.run(cmd, stdout=subprocess.PIPE, check=True)
            result_str = " * {} Cloned".format(repo_name)
            print(result_str)
            repo_result.append(result_str)
        except Exception:
            result_str = " * {} CANNOT be cloned".format(repo_name)
            print(result_str)
            repo_result.append(result_str)
        n += 1
    if Config.get_config().save:
        with open(Config.get_config().save, "w") as text_file:
            text_file.write("\n".join(repo_result))
            text_file.write("\n")
