from git_cloner.model.owner.owner import Owner
from git_cloner.model.repository import RawRepository
from git_cloner.utils import utils
from git_cloner.utils.config import Config


class GithubOwner(Owner):
    @staticmethod
    def build_from_config():
        conf = Config.get_config()
        owner, is_user = (conf.target_user, True) if conf.target_user else (conf.org, False)
        return GithubOwner(owner, is_user)

    REPOS_FROM_USER_TEMPLATE = "{base}/repos?page={{p}}&per_page=100"
    BASE_API_URL_USER = "https://api.github.com/users/{u}"
    BASE_API_URL_ORG = "https://api.github.com/orgs/{o}"

    def __init__(self, owner, is_user=True):
        self.is_user = is_user
        self.owner = owner

    def api_base_url(self):
        if self.is_user:
            return self.BASE_API_URL_USER.format(u=self.owner)
        return self.BASE_API_URL_ORG.format(o=self.owner)

    def get_repos_url(self):
        return self.REPOS_FROM_USER_TEMPLATE.format(base=self.api_base_url())

    def get_repos(self):
        return [RawRepository(repo) for repo in self.get_repos_rec(0, self.get_repos_url())]

    # Is not the best form but i'm, trying tailrec
    @staticmethod
    def get_repos_rec(p, url, acc=list()):
        limit = Config.get_config().limit
        size = len(acc)
        if p > 0 and size != limit * p:
            return acc
        else:
            return GithubOwner.get_repos_rec(
                p + 1, url,
                acc + utils.make_request(url.format(p=p), auth_if_possible=True))
