from uuid import uuid4

from git_cloner.model.remotes.remote import Remote


class GithubRemote(Remote):
    SSH_TEMPLATE = "git@github.com:{owner}/{repo_name}.git"
    HTTPS_TEMPLATE = "https://github.com/{owner}/{repo_name}.git"

    @staticmethod
    def build_from_url(url):
        name = url.split("/")[-1].replace(".git", "")
        ssh = url.startswith("git")
        if ssh:
            owner = url.split(":")[-1].split("/")[0]
        else:
            owner = url.split("/")[-2]
        return GithubRemote(owner, name, ssh)

    def __init__(self, owner, repo_name, is_ssh, name=uuid4()):
        self.ssh = is_ssh
        self.name = name
        self.repo_name = repo_name
        self.owner = owner

    def build_remote_url(self):
        if self.ssh:
            return self.SSH_TEMPLATE.format(owner=self.owner, repo_name=self.repo_name)
        return self.HTTPS_TEMPLATE.format(owner=self.owner, repo_name=self.repo_name)

    def get_name(self):
        return self.name

    def get_owner(self):
        return self.owner
