import subprocess
from pathlib import Path

import copy
import os
import shutil

from git_cloner.model.remotes.github_remote import GithubRemote
from git_cloner.utils.logger import *


class Repository(object):
    CMD_CHECK = "git -C {} rev-parse"
    CMD_INIT = "git init {}"

    GITLAB = "gitlab"
    GITHUB = "github"

    OPERATION_CLONE = "clone"

    @staticmethod
    def clone(remote, root_path):
        logger = get_logger_console(Repository)
        new_remote = copy.copy(remote)
        new_remote.name = "origin"
        clone_path = os.path.join(root_path, new_remote.repo_name)
        if Config.get_config().force:
            shutil.rmtree(clone_path, ignore_errors=True)
        elif os.path.exists(clone_path):
            logger.error("Error cloning: {}, path exists {}".format(remote.build_remote_url(), clone_path))
            log_result(remote, Repository.OPERATION_CLONE, False)
            return None
        result = subprocess.run(new_remote.build_clone_cmd().format(path=clone_path).split(" "))
        if result.returncode != 0:
            logger.error("Error cloning: {}".format(remote.build_remote_url()))
            log_result(remote, Repository.OPERATION_CLONE, False)
            return None
        log_result(remote, Repository.OPERATION_CLONE, True)
        return Repository(clone_path, [new_remote])

    def __init__(self, path, remotes=None):
        if remotes is None:
            self.remotes = {}
        else:
            self.remotes = remotes
        self.path = path
        self.name = Path(self.path).name

    def add_remote(self, remote):
        self.remotes.update({remote.name, remote})

    def init_git(self):
        subprocess.run(self.CMD_INIT.format(self.path).split(" "))

    def check_git(self):
        return subprocess.run(self.CMD_CHECK.format(self.path).split(" "),
                              stdout=subprocess.PIPE,
                              stderr=subprocess.PIPE).returncode == 0


class RawRepository(object):
    def __init__(self, raw_data):
        self.raw_data = raw_data

    def clone(self, path):
        owner = self.raw_data['owner']['login']
        repo_name = self.raw_data['name']
        remote = GithubRemote(owner, repo_name, Config.get_config().ssh)
        return Repository.clone(remote, path)
