import getpass

import os


# Recursive problems
def read(fname):
    with open(os.path.join(os.path.dirname(__file__), fname)) as f:
        return f.read().strip()


class Config(object):
    _KEY_TARGET_USER = "--target-user"
    _KEY_HOST = "--host"
    _KEY_TOKEN = "--priv-token"
    _KEY_TYPE = "--type"
    _KEY_USER = "--user"
    _KEY_PASS = "--pass"
    _KEY_ORG = "--org"
    _KEY_LIMIT = "--limit"
    _KEY_DIR_IN = "--in"
    _KEY_DIR_OUT = "--out"
    _KEY_FORCE = "--force"
    _KEY_SAVE = "--save"
    _KEY_SSH = "--ssh"
    _KEY_VERBOSE = "--verbose"
    _KEY_RETRY = "--retry"
    instance = None

    def __init__(self, user, password, org, limit, dir_in, dir_out,
                 force, save, ssh, repo_type, token, host, target_user, verbose, retry):
        self.retry = retry
        self.target_user = target_user
        self.user = user
        self.password = password
        self.org = org
        self.limit = limit
        self.dir_in = dir_in
        self.dir_out = dir_out
        self.force = force
        self.save = save
        self.ssh = ssh
        self.type = repo_type
        self.token = token
        self.host = host
        self.verbose = verbose

    @classmethod
    def load_config(cls, arguments):
        user = arguments.pop(cls._KEY_USER, None)
        password = arguments.pop(cls._KEY_PASS, None)
        org = arguments.pop(cls._KEY_ORG, None)
        dir_in = arguments.pop(cls._KEY_DIR_IN, ".")
        dir_out = arguments.pop(cls._KEY_DIR_OUT, ".")
        force = arguments.pop(cls._KEY_FORCE, False)
        save = arguments.pop(cls._KEY_SAVE, None)
        ssh = arguments.pop(cls._KEY_SSH, False)
        repo_type = arguments.pop(cls._KEY_TYPE, None)
        host = arguments.pop(cls._KEY_HOST, None)
        path = arguments.pop(cls._KEY_TOKEN, None)
        verbose = arguments.pop(cls._KEY_VERBOSE, False)
        token = read(path) if path is not None else None
        target_user = arguments.pop(cls._KEY_TARGET_USER, None)
        retry = arguments.pop(cls._KEY_RETRY, None)
        if user is not None and password is None:
            password = getpass.getpass()

        cls.instance = Config(user, password, org, 100, dir_in, dir_out,
                              force, save, ssh, repo_type, token, host, target_user, verbose, retry)
        return cls.instance

    @classmethod
    def get_config(cls):
        return cls.instance if cls.instance is not None else cls.load_config({})
