import unittest
from mock_decorators.function_mock import FunctionMock

from git_cloner.utils import utils
from git_cloner.model.owner.github_owner import GithubOwner
from git_cloner.utils.config import Config
from tests.git_cloner.utils.gen_utils import *

list_repos = [generate_repo_github() for _ in range(gen_int(101, 501))]


class GithubOwnerTest(unittest.TestCase):
    def test_base_api_url_user(self):
        owner = GithubOwner("fhuertas", True)
        self.assertEqual(owner.api_base_url(), "https://api.github.com/users/fhuertas")

    def test_base_api_url_org(self):
        owner = GithubOwner("fhuertas", False)
        self.assertEqual(owner.api_base_url(), "https://api.github.com/orgs/fhuertas")

    def test_get_repos_url(self):
        owner = GithubOwner("fhuertas", False)
        self.assertEqual(owner.get_repos_url(), "https://api.github.com/orgs/fhuertas/repos?page={p}&per_page=100")

    def test_get_all_repos(self):
        self.count = 0
        Config.load_config({})

        def get_repos(*args, **kwargs):
            url = args[0]
            init = 100 * self.count
            end = 100 * (self.count + 1)
            self.assertEqual(
                url,
                "https://api.github.com/users/fhuertas/repos?page={p}&per_page=100".format(p=self.count))
            self.count += 1
            return list_repos[init:end]

        @FunctionMock(utils, 'make_request', get_repos, False)
        def inner_test():
            owner = GithubOwner("fhuertas", True)
            return owner.get_repos()

        result = inner_test()
        self.assertEqual(len(result), len(list_repos))

    def test_get_all_repos_no_result(self):
        self.count = 0
        Config.load_config({})

        def get_repos(*args, **kwargs):
            url = args[0]
            self.assertEqual(
                url,
                "https://api.github.com/users/fhuertas/repos?page={p}&per_page=100".format(p=self.count))
            self.count += 1
            return []

        @FunctionMock(utils, 'make_request', get_repos, False)
        def inner_test():
            owner = GithubOwner("fhuertas", True)
            return owner.get_repos()

        result = inner_test()
        self.assertEqual(len(result), 0)
